######################################
# This is an attempt to separate the functions to evaluate the results of a given model of the Workflow

# The idea is to define a Evaluator class that will be used by PrecisionAnalysis to discriminate the results
# of the experiments running, so the model specific parts can be removed from AutoRPE
#
#


class Evaluator:
    """
    The purpose of the evaluator class is to wrap functions to evaluate a given result for a given model.
    This is only an example structure, which should be  extended with a subclass for a given case.

    The original idea is that the evaluator class will have two methods:
    -Evaluate success: Binary answer about the accuracy of the results (True of False).
    -Evaluate fitness: Real value that measures how good a result is (the lower the better).

    Additionally, it was handy to have a plot function to visualize the results.

    The idea is that in order to use the evaluator a result to evaluate will be passed as an argument.
    We don't assume the type of result.

    Since the evaluator should not be used, but only be extended with a subclass, its methods will raise a
    NotImplemented in case the method hasn't been properly extended.
    """
    def __init__(self):
        pass

    def evaluate_success(self, result):
        raise NotImplemented("No evaluate_success function has been provided.")

    def evaluate_fitness(self, result):
        raise NotImplemented("No evaluate_fitness function has been provided.")

    def plot(self, result):
        raise NotImplemented("No plot function has been provided.")


if __name__ == "__main__":
    """
    Here we can find two simple examples of how one can define an evaluator that will be later used by
    """


    def test_evaluator(results_to_evaluate, evaluator):
        """
        Simple function that will try to use the evaluator with a list of sample results.
        """
        for result in results_to_evaluate:
            print()
            print("Result: %f" % result)
            print("Success: %s" % evaluator.evaluate_success(result))
            print("Fitness: %.2f" % evaluator.evaluate_fitness(result))


    class StandaloneEvaluator(Evaluator):
        """
        This is an example of an evaluator that does not require any prior data to evaluate a result.

        For example, here we will expect a real number, and we will consider the results accurate if the result is between
        the (0,10) range, and the fitness will be the difference between the result provided and 5.
        """
        def __init__(self):
            super().__init__()

        def evaluate_success(self, result):
            return 0. < result < 10.

        def evaluate_fitness(self, result):
            return abs(result-5.)


    class DataEvaluator(Evaluator):
        """
        Here we will do a similar example but in this case, we will provide the range of success and the value for which
        the fitness is 0.
        """

        def __init__(self, data):
            super().__init__()
            self.data = data

        def evaluate_success(self, result):
            """
            In this case, we will provide the range as the first argument of a list
            """
            bot, top = self.data[0]
            return bot < result < top

        def evaluate_fitness(self, result):
            """
             ... and the target value as the second argument of a list,
            """
            return abs(result - self.data[1])


    # Here we will try both examples.
    # First we define some "results" that will be evaluated.
    results_to_evaluate = [-1., 1., 10, 9.99, 100]

    # We define a standalone evaluator.
    # In this case we don't need to provide any argument at the moment of initializing the object.
    standalone_evaluator = StandaloneEvaluator()

    # We can test the evaluator with the results provided
    test_evaluator(results_to_evaluate, standalone_evaluator)


    # We define now a data evaluator

    # We define the data that will be used to initialize the evaluator

    #        bot top   target
    data = [ (0., 10.), 5.]

    # We initialize the evaluator using the data
    data_evaluator = DataEvaluator(data)

    # We try the evaluator with the results provided
    test_evaluator(results_to_evaluate, data_evaluator)
